@@ -2,23 +2,49 @@ from datetime import datetime
 from couchdbkit.ext.django.schema import *
 from couchdbkit.exceptions import ResourceNotFound
 from django.conf import settings
-from django.contrib.auth.models import get_hexdigest, check_password, UNUSABLE_PASSWORD
+from django import VERSION as django_version
 from django.core.exceptions import ImproperlyConfigured
 from django.core.mail import send_mail
 import random
 
-from . import app_label
+from django_couchdb_utils.auth import app_label
+
+"""
+ django versions < 1.4 backward-compatibly patches
+"""
+
+if django_version[:2] > (1, 3):
+    # > 1.3.x
+    from django.contrib.auth.hashers import make_password, check_password, UNUSABLE_PASSWORD
+    def set_password(self, raw_password):
+        self.password = make_password(raw_password)
+else:
+    # < 1.4.x
+    from django.contrib.auth.models import get_hexdigest, check_password, UNUSABLE_PASSWORD
+    def set_password(self, raw_password):
+        algo = 'sha1'
+        salt = get_hexdigest(algo, str(random.random()), str(random.random()))[:5]
+        hsh = get_hexdigest(algo, salt, raw_password)
+        self.password = '%s$%s$%s' % (algo, salt, hsh)
 
 
 class SiteProfileNotAvailable(Exception):
     pass
 
 
+class UsernameException(Exception):
+    pass
+
+
+class PasswordException(Exception):
+    pass
+
+
 class User(Document):
     username      = StringProperty(required=True)
     first_name    = StringProperty(required=False)
     last_name     = StringProperty(required=False)
-    email         = StringProperty(required=False)
+    email         = StringProperty(required=True)
     password      = StringProperty(required=True)
     is_staff      = BooleanProperty(default=False)
     is_active     = BooleanProperty(default=True)
@@ -40,9 +66,9 @@ class User(Document):
 
     def save(self):
         if not self.check_username():
-            raise Exception('This username is already in use.')
+            raise UsernameException('The username %s is already in use.' % self.username)
         if not self.check_email():
-            raise Exception('This email address is already in use.')
+            raise PasswordException('The email address %s is already in use.' % self.email)
         return super(User, self).save()
 
 
@@ -72,10 +98,7 @@ class User(Document):
         return True
 
     def set_password(self, raw_password):
-        algo = 'sha1'
-        salt = get_hexdigest(algo, str(random.random()), str(random.random()))[:5]
-        hsh = get_hexdigest(algo, salt, raw_password)
-        self.password = '%s$%s$%s' % (algo, salt, hsh)
+        set_password(self, raw_password)
 
     def check_password(self, raw_password):
         """
@@ -136,14 +159,19 @@ class User(Document):
 
     @classmethod
     def get_user(cls, username, is_active=True):
-        param = {"key": username}
+        user = None
+
+        if not user:
+            res = cls.view('django_couchdb_utils_auth/users_by_username',
+                     include_docs = True,
+                     reduce       = False,
+                     key          = username,
+                )
+            user = res.first()
+
+        if user and (not is_active or user.is_active):
+            return user
 
-        r = cls.view('django_couchdb_utils_auth/users_by_username',
-                     include_docs=True,
-                     reduce=False,
-                     **param).first()
-        if r and (not is_active or r.is_active):
-            return r
         return None
 
     @classmethod
@@ -170,7 +198,7 @@ class User(Document):
         view = cls.view('django_couchdb_utils_auth/users_by_username',
                 reduce=True,
             )
-        return view.first()['value']
+        return view.first()['value'] if view else 0
 
 
 class UserProfile(Document):
