@@ -33,5 +33,5 @@ urlpatterns = patterns('',
                            direct_to_template,
                            {'template': 'registration/registration_closed.html'},
                            name='registration_disallowed'),
-                       (r'', include('registration.auth_urls')),
+                       (r'', include('django_couchdb_utils.registration.auth_urls')),
                        )
